from __future__ import print_function
from datetime import datetime
from dateutil.parser import parse
from argparse import ArgumentParser


def get_arguments():
    a = ArgumentParser(description="Perform various operations on dates and date ranges.")

    a.add_argument('start', metavar='START', help="the start date")
    a.add_argument('end', nargs='?', metavar='END', help="the end date")

    a.add_argument('-y', '--years', action='store_const', dest='unit', const='years', help='show difference in years')
    a.add_argument('-m', '--months', action='store_const', dest='unit', const='months', help='show difference in months')
    a.add_argument('-w', '--weeks', action='store_const', dest='unit', const='weeks', help='show difference in weeks')
    a.add_argument('-d', '--days', action='store_const', dest='unit', const='days', help='show difference in days')
    a.add_argument('-H', '--hours', action='store_const', dest='unit', const='hours', help='show difference in hours')
    a.add_argument('-M', '--minutes', action='store_const', dest='unit', const='minutes', help='show difference in minutes')
    a.add_argument('-S', '--seconds', action='store_const', dest='unit', const='seconds', help='show difference in seconds')
    a.add_argument('-u', '--microseconds', action='store_const', dest='unit', const='microseconds', help='show difference in microseconds')
    a.add_argument('-b', '--business-days', action='store_const', dest='unit', const='business_days', help='show difference in business days')

    a.add_argument('--holiday', metavar='DATE', action='append', help='holiday to include')
    a.add_argument('--holiday-file', metavar='FILE', help='holidays to include from a file')

    return a.parse_args()


def main():
    args = get_arguments()
    start_dt = parse(args.start)
    end_dt = datetime.now if not args.end else parse(args.end)
    kwargs = {}
    if args.unit == 'business_days':
        holidays = []
        if args.holiday:
            holidays.extend(parse(h) for h in args.holiday)
        if args.holiday_file:
            holidays.extend(parse(l) for l in open(args.holiday_file))
        kwargs['holidays'] = holidays
    print(__import__(args.unit)(end_dt, start_dt, **kwargs))



from __future__ import print_function
from . import TIME_UNITS, date_range, increment
from dateutil.parser import parse
from datetime import datetime
from argparse import ArgumentParser


def get_arguments():
    a = ArgumentParser(description="Increment/decrement a date by some unit(s) of time.")

    a.add_argument('dates', metavar='DATE', nargs='*', help='a date to increment/decrement')

    a.add_argument('-F', '--format', default='%Y-%m-%d %H:%M:%S', help='use STRING as output format')
    a.add_argument('-I', '--iterate', default=False, action='store_true', help='expand a date range')

    a.add_argument('-y', '--years', metavar='NUM', type=int, default=0, help='add NUM years to date(s)')
    a.add_argument('-m', '--months', metavar='NUM', type=int, default=0, help='add NUM months to date(s)')
    a.add_argument('-w', '--weeks', metavar='NUM', type=int, default=0, help='add NUM weeks to date(s)')
    a.add_argument('-d', '--days', metavar='NUM', type=int, default=0, help='add NUM days to date(s)')
    a.add_argument('-H', '--hours', metavar='NUM', type=int, default=0, help='add NUM hours to date(s)')
    a.add_argument('-M', '--minutes', metavar='NUM', type=int, default=0, help='add NUM minutes to date(s)')
    a.add_argument('-S', '--seconds', metavar='NUM', type=int, default=0, help='add NUM seconds to date(s)')
    a.add_argument('-u', '--microseconds', metavar='NUM', type=int, default=0, help='add NUM microseconds to date(s)')
    a.add_argument('-b', '--business-days', metavar='NUM', type=int, default=0, help='add NUM business days to date(s)')

    a.add_argument('--holiday', metavar='DATE', action='append', help='holiday to include')
    a.add_argument('--holiday-file', metavar='FILE', help='holidays to include from a file')

    args = a.parse_args()
    kwargs = dict((k, v) for k, v in vars(args).items() if k in TIME_UNITS)
    return args, kwargs


def main():
    args, kwargs = get_arguments()
    # Only care about holidays when looking at business days
    if args.business_days:
        holidays = []
        if args.holiday:
            holidays.extend(parse(h) for h in args.holiday)
        if args.holiday_file:
            holidays.extend(parse(l) for l in open(args.holidays_file))
        kwargs['holidays'] = holidays
    # If iterating, the first two arguments must be start and end dates
    if args.iterate and len(args.dates) == 2:
        start_dt = parse(args.dates[0])
        end_dt = parse(args.dates[1])
        dates = list(date_range(start_dt, end_dt, **kwargs))
    else:
        if args.dates:
            dates = [parse(dt) for dt in args.dates]
        else:
            dates = [datetime.now()]
        if any(bool(v) for v in kwargs.values()):
            dates = [increment(dt, **kwargs) for dt in dates]
    for dt in dates:
        print(dt.strftime(args.format))



import calendar, pytz, math
from datetime import datetime, date, timedelta
from dateutil.relativedelta import relativedelta

TIME_UNITS = [
    'business_days',
    'years',
    'months',
    'weeks',
    'days',
    'hours',
    'minutes',
    'seconds',
    'microseconds'
    ]

QUARTER_MONTHS = [10, 1, 4, 7]


def timezone(dt, timezone='utc'):
    """Add timezone information to a naive datetime."""
    timezone = pytz.timezone(timezone)
    return datetime(tzinfo=timezone, *dt.timetuple()[:6])


def timezone_convert(dt, timezone):
    """Convert aware datetime to a different timezone."""
    timezone = pytz.timezone(timezone)
    return dt.astimezone(timezone)


def increment(dt, business_days=0, holidays=[], **inc):
    """Increment a date by the given amount.
    Arguments:
        dt -- the date to increment
    Keyword arguments:
        holidays -- list of holiday dates
        business_days -- number of business days to increment
        years -- number of years to increment
        months -- number of months to increment
        weeks -- number of weeks to increment
        days -- number of days to increment
        hours -- number of hours to increment
        minutes -- number of minutes to increment
        seconds -- number of seconds to increment
        microseconds -- number of microseconds to increment
    """
    new_dt = dt + relativedelta(**inc)
    if business_days != 0:
        i = business_days / abs(business_days)
        while business_days != 0:
            while True:
                new_dt = increment(new_dt, days=i)
                if is_business_day(new_dt, holidays):
                    break
            business_days -= i
    return new_dt


def is_business_day(dt, holidays=[]):
    if dt.weekday() in (calendar.SATURDAY, calendar.SUNDAY):
        return False
    if holidays and dt in holidays:
        return False
    return True


def date_range(start_dt, end_dt, holidays=[], **inc):
    """Generate a range of dates/datetimes based on the given increment."""
    # If incrementing by business days, make sure we start on one
    if inc.get('business_days', 0) and not is_business_day(start_dt, holidays=holidays):
        cur_dt = increment(start_dt, business_days=1, holidays=holidays)
    else:
        cur_dt = start_dt
    while cur_dt <= end_dt:
        yield cur_dt
        prev_dt = cur_dt
        cur_dt = increment(cur_dt, **inc)
        if cur_dt == prev_dt:
            break


def month_start(dt):
    """Get the beginning of the month for a given date."""
    return date(*dt.timetuple()[:2]+(1,))


def month_end(dt):
    """Get the end of the month for a given date."""
    return month_start(dt) - timedelta(days=1)


def quarter(dt):
    """Get the quarter for a given date."""
    quarter_months = [range(r, r+3) for r in (i for i in QUARTER_MONTHS)]
    for qm in quarter_months:
        if dt.month in qm:
            return quarter_months.index(qm) + 1


def quarter_start(dt):
    """Get the beginning of the quarter for a given date."""
    m = QUARTER_MONTHS.index(quarter(dt)-1) + 2
    return date(dt.year, m, 1)


def quarter_end(dt):
    """Get the end of the quarter for a given date."""
    m = QUARTER_MONTHS.index(quarter(dt)-1) + 2
    return month_end(date(dt.year, m, 1))


def day_of_year(dt):
    return dt.timetuple()[7]


def microseconds(end_dt, start_dt):
    d = end_dt - start_dt
    return (d.days*24*60*60*1000000)+(d.seconds*1000000)


def seconds(end_dt, start_dt):
    d = end_dt - start_dt
    return (d.days*24*60*60)+d.seconds


def minutes(end_dt, start_dt):
    d = end_dt - start_dt
    return (d.days*24*60)+(d.seconds/60)


def hours(end_dt, start_dt):
    d = end_dt - start_dt
    return (d.days*24)+(d.seconds/60/60)


def days(end_dt, start_dt):
    return (end_dt - start_dt).days


def weeks(end_dt, start_dt):
    return int(math.ceil(days(end_dt, start_dt)/7.0))


def months(end_dt, start_dt):
    return days(end_dt, start_dt)/31


def years(end_dt, start_dt):
    return months(end_dt, start_dt)/12


def business_days(end_dt, start_dt, holidays=[]):
    return len(list(date_range(start_dt, end_dt, business_days=1, holidays=holidays)))
